% Constructor of the finite element block class to be used with a
% grounding spring/damper elements.
%
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Parameters: those recognized by feblock
% Optional parameters:
%  translation_stiffness_matrix=  cell array of stiffness matrices,  one
%  per gcell
%  rotation_stiffness_matrix=  cell array of stiffness matrices,  one
%  per gcell
%  rotation_damping_matrix=  cell array of damping matrices,  one
%  per gcell
%  translation_damping_matrix=  cell array of damping matrices,  one
%  per gcell
% 
% Example:
% e1,e2,e3= ortho normal basis vectors
% Stiffness matrix for constraints tying a node to the curve  defined by
% e1: 
%         Gstiffness{end+1}=K*(e2*e2'+e3*e3'); 
% Damping matrix  that generates  viscous torque for angular velocity around
% vector e1
%         Gdamping{end+1}=D*(e1*e1'); 
function retobj = feblock_defor_ground (varargin)
    class_name='feblock_defor_ground';
    if (nargin == 0)
        parent = feblock;
        self.translation_stiffness_matrix=  {};
        self.rotation_stiffness_matrix=  {};
        self.translation_damping_matrix=  {};
        self.rotation_damping_matrix=  {};
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            parent = feblock(varargin{:});
            self.translation_stiffness_matrix ={};
            if (isfield(Parameters,'translation_stiffness_matrix'))
                self.translation_stiffness_matrix = Parameters.translation_stiffness_matrix;
            end
            self.rotation_stiffness_matrix ={};
            if (isfield(Parameters,'rotation_stiffness_matrix'))
                self.rotation_stiffness_matrix = Parameters.rotation_stiffness_matrix;
            end
            self.translation_damping_matrix ={};
            if (isfield(Parameters,'translation_damping_matrix'))
                self.translation_damping_matrix = Parameters.translation_damping_matrix;
            end
            self.rotation_damping_matrix ={};
            if (isfield(Parameters,'rotation_damping_matrix'))
                self.rotation_damping_matrix = Parameters.rotation_damping_matrix;
            end
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
