% Compute the stiffness matrices of the individual entities in the block
% (in this case nodes).
% Return  element matrix set so they may be assembled.
%    Call as
% function ems = stiffness (self, geom, u)
%     geom=geometry field
%     u=displacement field
%
function ems = stiffness (self, geom, u)
    gcells = get(self.feblock,'gcells');
    dim = get(geom,'dim');
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    tangents =eye(dim);
    % Material
    mat = get(self.feblock, 'mater');
    D = tangent_moduli(mat,struct([]));
    [Princd,W]=eig(D);
    [ignore,ix]=sort(diag(W));
    Wmean=(W(ix(1),ix(1))+W(ix(2),ix(2)))/2;
    if Wmean<=0
        Wmean=max(diag(W));
    end
    %     Evaluate the nodal basis function gradients
    bfun_gradients = nodal_bfun_gradients (self, geom);
    %     Preallocate
    Kes= cell(length(bfun_gradients),1);
    eqnums = cell(length(bfun_gradients),1);
    for i=1:length(bfun_gradients)
        eqnums{i} =[];
        Kes{i} = [];
    end
    xs =gather(geom, 1:length(bfun_gradients), 'values', 'noreshape'); % coordinates of all nodes
    eqnumss =gather(u, 1:length(bfun_gradients), 'eqnums','noreshape');% equation numbers for all nodes
    % Now loop over all finite element nodes
    iem =1;
    for nix=1:length(bfun_gradients)
        if bfun_gradients{nix}.Vpatch>0 % This node has a element patch in this block
            np=length(bfun_gradients{nix}.patchconn);
            c = xs(nix, :); % coordinates of central node
            D = tangent_moduli (mat, struct ('xyz',c));% material tangent
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,tangents);%RAW: tangent vectors?
            end
            Bnodal= self.hBlmat (self,bfun_gradients{nix}.Nspd,c,Rm);
            Ke = Bnodal'*D*Bnodal * bfun_gradients{nix}.Vpatch;
            if (self.stabfact>0)
                lx = xs(bfun_gradients{nix}.patchconn,:); % coordinates of nodes
                Phi =self.hPhi(self,dim,np,lx,c);
                A =Phi'*Phi;
                try
                    A1 =Phi* ((Phi'*Phi)\Phi');
                    Kstab=self.stabfact*Wmean*(eye(size(Ke))-A1);
                    Ke = Ke + Kstab;
                catch
                    warning
                end
            end
            if (self.irrot_stabfact>0)
                S=skewmat(zeros(dim,1));
                Vrot = repmat(S, 1, (np)); % preallocate
                for kK= 1:(np)
                    Vrot(:,(kK-1)*dim+1:kK*dim) =skewmat(bfun_gradients{nix}.Nspd(kK,:));
                end
                Kstab=self.irrot_stabfact*Vrot'*Vrot* bfun_gradients{nix}.Vpatch;
                Ke = Ke + Kstab;
            end
            Kes{iem} =  Ke;
            eqnums{iem} =  reshape(eqnumss(bfun_gradients{nix}.patchconn,:)',(np)*size(eqnumss,2),1);
            iem=iem+1;
        end
    end
    ems  = elematset(struct('mat',{Kes(1:iem-1)}, 'eqnums',{eqnums(1:iem-1)}));
end


