% Compute the damping matrices of the individual gcells.
%
% function ems = damping (self, geom, u)
%
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness(feb, geom, u)
%     geom=geometry field
%     u=displacement field
%
function ems = damping (self, geom, u)
    gcells = get(self,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self, 'integration_rule');
    % - volumetric terms
    pc_v = get(integration_rule, 'param_coords',1);
    w_v  = get(integration_rule, 'weights',1);
    n_v = get(integration_rule, 'npts',1);
    % - shear terms
    pc_s = get(integration_rule, 'param_coords',2);
    w_s  = get(integration_rule, 'weights',2);
    n_s = get(integration_rule, 'npts',2);
    Blmat =get(self,'Blmat');
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self, 'mater');
    % Loop over all the geometry cells
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Ke{i} =zeros(get(geom,'dim')*nfens);
    end
    xs =get(geom,'values');
    Blmat =get( self,'Blmat');
    for j=1:n_v
        N = bfun(gcells,pc_v(j,:));
        Nder = bfundpar(gcells,pc_v(j,:));
        for i=1:size(conns,1)
            conn =conns(i,:);
            x=xs(conn,:);
            c =N'*x;% physical location
            J = x' * Nder;%J = Jacobian_matrix(gcells,Nders{j},x);%J = x' * Nder;
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            Jac = Jacobian_volume(gcells,conn, N, J, x);
            Ndersp = Nder/(Rm'*J);% derivatives wrt local coor
            B = Blmat(self,N,Ndersp,c,Rm);% strain-displacement
            D = damping_moduli (mat,struct('kind',self.split));
            Ke{i} = Ke{i} + (B'*(D*(Jac*w_v(j)))*B);
        end
    end
    for j=1:n_s
        N = bfun(gcells,pc_s(j,:));
        Nder = bfundpar(gcells,pc_s(j,:));
        for i=1:size(conns,1)
            conn =conns(i,:);
            x=xs(conn,:);
            c =N'*x;% physical location
            J = x' * Nder;%J = Jacobian_matrix(gcells,Nders{j},x);%J = x' * Nder;
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            Jac = Jacobian_volume(gcells,conn, N, J, x);
            Ndersp = Nder/(Rm'*J);% derivatives wrt local coor
            B = Blmat(self,N,Ndersp,c,Rm);% strain-displacement
            D = damping_moduli (mat, ...
                struct('kind',[self.split '_shear']));
            Ke{i} = Ke{i} + (B'*(D*(Jac*w_s(j)))*B);
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
    
end
