% Compute the capacity matrices of the individual gcells.
%
% function ems = capacity(self, geom, temp)
%
% Return an array of them so they may be assembled.
%    Call as
%       ems = capacity(feb, geom, temp)
%    where
%       geom=geometry field
%       temp=temperature field
%
function ems = capacity(self, geom, temp)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    cv = get(mat,'specific_heat');
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Ce = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(temp,conns(i,:),'eqnums');
        Ce{i} =zeros(nfens);
    end
    xs =get(geom,'values');
     % Now loop over all gcells in the block
     for i=1:size(conns,1)
         conn =conns(i,:);
         x=xs(conn,:);
         for j=1:npts_per_gcell
             J = Jacobian_matrix(gcells,Nders{j},x);
             Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
             Ce{i} = Ce{i} + Ns{j}*(Ns{j}' * (cv*Jac * w(j)));
         end
    end
    ems  = elematset(struct('mat',{Ce}, 'eqnums',{eqnums}));
end
