% Compute the source-term load vectors of the individual gcells.
%
% function evs = source_loads(self, geom, temp)
%
% Return an array of them so they may be assembled.
%    Call as
%       evs = source_loads(feb, geom, temp)
%    where
%       geom=geometry field
%       temp=temperature field
%
function evs = source_loads(self, geom, temp)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Material
    mat = get(self.feblock, 'mater');
    Q = get(get(mat,'property'),'source');
    evs  = elevecset(struct('vec',{{}}, 'eqnums',{{}}));
    if (~ isempty(Q)) % Any loads due to internal heat sources?
        if (strcmp(class(Q),'function_handle'))
            need_to_eval=1;
        else
            need_to_eval=0;
        end
        % Now loop over all gcells in the block
        conns = get(gcells, 'conn'); % connectivity
        Fe = cell(size(conns,1),1);
        eqnums = cell(size(conns,1),1);
        for i=1:size(conns,1)
            eqnums{i} =gather(temp,conns(i,:),'eqnums');
            Fe{i} =zeros(nfens,1);
        end
        xs =get(geom,'values');
        for j=1:npts_per_gcell
            N = bfun(gcells,pc(j,:));
            Nder = bfundpar(gcells,pc(j,:));
            for i=1:size(conns,1)
                conn =conns(i,:);
                x=xs(conn,:);
                J = Jacobian_matrix(gcells,Nder,x);
                Jac = Jacobian_volume(gcells,conn, N, J, x);
                if (need_to_eval)
                    val = feval(Q,N'*x);
                else
                    val = Q;
                end
                Fe{i} = Fe{i} + N *  (val * Jac * w(j));
            end
        end
        evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
    end
end
