% Compute the element load vectors corresponding to surface heat transfer.
%
% function evs = surface_transfer_loads (self, geom, temp, amb)
%
% Return an array of the element vectors so they may be assembled.
%     geom=geometry field
%     temp=temperature field
%     amb = ambient temperature field
function evs = surface_transfer_loads (self, geom, temp, amb)
    ems = surface_transfer(self, geom, temp);
    gcells = get(self.feblock,'gcells');
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(temp,conns(i,:),'eqnums');
        Fe{i} =zeros(length(eqnums{i}),1);
    end
    mat=get(ems,'mat');
    for i=1:size(conns,1)
        conn = conns(i,:); % connectivity
        pT = gather(amb, conn, 'prescribed_values');
        if norm (pT) ~= 0
            Fe{i} =mat{i}*pT;
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
