% Generate connectivity matrices of the individual nodes in the block
%    Call as
% gcells = genconn(feb)
%     x=geometry field
%     u=displacement field
%
function ngcells = genconn (self)
    gcells = get(self.feblock,'gcells');
    conns = get(gcells, 'conn'); % connectivity
    % Make the inverse map from finite element nodes to geometric cells
    f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
    gmap=get (f2gmap,'gcell_map');
    % Pre-allocate the gcell matrices
    conns = cell(length(gmap),1);
    iem=0;
    % Now loop over all finite element nodes
    maxnfens=0;
    for nix=1:length(gmap)
        gl=gmap{nix};
        thisnn=nix;
        if ~isempty(gl) % This node has a element patch in this block
            % establish local numbering of all nodes of the patch @node nix
            lnmap =0*(1:length(gmap));
            kconns = get(subset(gcells,gl), 'conn'); % connectivity
            for k=1:length(gl)
                kconn = kconns(k,:); % connectivity
                for j= 1:length(kconn)
                    if (lnmap(kconn(j)) == 0) && (kconn(j) ~= thisnn)
                        lnmap(kconn(j))=kconn(j);
                    end
                end
            end
            patchconn=lnmap(lnmap>0);
            iem=iem+1;
            conns{iem} = [patchconn thisnn];
            maxnfens =max( [maxnfens, length(patchconn)+1]);
        end
    end
    conn =zeros(length(conns),maxnfens,'int32');
    for k=1:size(conn,1)
        c=conns{k};
        conn(k,1:length(c)) =c;
        conn(k,length(c)+1:end) =c(end);
    end 
    ngcells = gcellset_N (struct('nfens',maxnfens,'conn',conn));
end


