% Constructor of the graphic viewer class.
%
% function retobj = graphic_viewer (varargin)
%
% See discussion of constructors in <a href="matlab:helpwin 'WFEM/classes/Contents'">classes/Contents</a>. 
%
% Parameters: 
%    axes= Matlab axes handle (optional). If the 'axes' parameter is not
%    supplied a new figure is created with new axes and those are used by
%    the graphic viewer.
%
function retobj = graphic_viewer (varargin)
    class_name='graphic_viewer';
    if (nargin == 0)
        parent=classbase;
        self.axes =[];
        self.pk_defaults =0;
        self.limits=[];
        h=gv_default_figure_('visible','off');
        self.snap_points =[];
        self = class(self, class_name, parent);
        retobj = self;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            parent=classbase;
            self.axes =0;
            if (isfield(Parameters,'axes' ))
                self.axes =Parameters.axes;
            else
                h=gv_default_figure_('visible','off');
                self.axes =[];
            end
            self.pk_defaults =0;
            if (isfield(Parameters,'pk_defaults' ))
                self.pk_defaults =Parameters.pk_defaults;
            end
            self.limits=[];
            self.snap_points =[];
            if (isfield(Parameters,'snap_points' ))
                self.snap_points =Parameters.snap_points;
            end
            self = class(self, class_name, parent);
            retobj = self;
        end
    else
        error('Illegal arguments!');
    end
    return;
end

function h=gv_default_figure_(varargin)
    h=figure(varargin{:});
end
