% Constructor  for electrostatics properties of materials.
%
% function retobj = property_electrostatics (varargin)
%
% This class represents  electrostatics properties of materials.
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>. 
% Parameters:
%     electrical_conductivity=matrix of conductivities; 2x2 for 2-D domains, 3x3
%                  for 3-D domains; the components are given in the local
%                  coordinate frame (feblock decides what that frame is).
%                  Electrical conductivity is typically a function of
%                  temperature, and hence it is also possible to supply this 
%                  property as a handle to a function that takes 
%                  the temperature as argument and returns the conductivity matrix.
%
function retobj = property_electrostatics (varargin)
    class_name='property_electrostatics';
    parent=property;
    if (nargin == 0)
        self.electrical_conductivity = 1.0;
        retobj = class(self,class_name, parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            self.electrical_conductivity = Parameters.electrical_conductivity;
            if strcmp( class(self.electrical_conductivity),'double')
                if (det(self.electrical_conductivity) < 0)
                    error('Negative determinant of electrical_conductivity!');
                end
            end
            retobj = class(self,class_name, parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
