% Advance the state by specified number of steps
%   Call as:
%      advance(obj, torque, dt, nsteps)
%   where
%      obj       - object
%      torque    - handle to function defining spatial torque
%      dt        - time step
%      nsteps    - number of steps to take
%
%   The first form returns the list of property names as a string,
%   the second form returns the value of the named property.
%
function retobj = advance(self,torque,dt,nsteps)
    Omegan1 = self.Omega;
    Rn1 = self.R;
    I=self.I;
%     in1=Rn1*I*Rn1';% debug
%     on1 = Rn1*Omegan1;% debug
%     Omegamid=I\(rotmat(dt/2*Omegan1)*I*Omegan1);
%     Omegamid'*I*Omegamid/2
    t=get(self.rotint,'t');
    Ptol=100*eps;
    for i=1:nsteps
        Tn1=Rn1'*feval(torque, t, Rn1);
        Pin1=I*Omegan1;
        DRn=rotmat(dt/2*Omegan1)*rotmat(dt/2*Omegan1); % predictor
        Pin=DRn'*(Pin1+dt/2*Tn1)+dt/2*Tn1; % predictor
        i=0; maxi=150;
        while (1)
            pPin=Pin;
            Omegan=I\Pin;
            DRn=reortho(rotmat(dt/2*Omegan1)*rotmat(dt/2*Omegan));
            Rn=Rn1*DRn;
            Tn=Rn'*feval(torque, t+dt, Rn);
            Pin=DRn'*(Pin1+dt/2*Tn1)+dt/2*Tn;
            i=i+1;
            if (i > maxi)
                if (max(abs(Pin-pPin)) > 1e-9)
                    warning(['trapez - Failed to converge: ||residual||=' num2str(norm(Pin))]);
                end
                break;
            end
            if (norm(Pin-pPin) < Ptol)
                break;
            end
        end
        % swap variables for next step
        Rn1=Rn;
        Omegan1=Omegan;
        t=t+dt;
    end
%     in=Rn*I*Rn';% debug
%     on=Rn*Omegan;
%     %    on'*in*on/2-on1'*in1*on1/2
%     on'*in*on/2
    self.R = Rn;
    self.Omega = Omegan;
    self.rotint = set(self.rotint,'t',t);
    retobj=self;
end
