% Advance the state by specified number of steps
%   Call as:
%      advance(obj, torque, dt, nsteps)
%   where
%      obj       - object
%      torque    - handle to function defining spatial torque
%      dt        - time step
%      nsteps    - number of steps to take
%
%   The first form returns the list of property names as a string,
%   the second form returns the value of the named property.
%
function retobj = advance(self,torque,dt,nsteps)
    Omegan1 = self.Omega;
    Rn1 = self.R;
    t=get(self.rotint,'t');
    I=self.I;
    % calculate effective loads at t_n-1
    Alphan1=I\(Rn1'*feval(torque, t, Rn1) - skewmat(Omegan1)*I*Omegan1);
    for i=1:nsteps
        % update rotation
        Thetan=dt*Omegan1+dt^2/2*Alphan1;
        Rn=Rn1*rotmat(skewmat(Thetan));
        % calculate effective loads at t_n
        Tn = Rn'*feval(torque, t+dt, Rn);
        Alphan = solve_Alpha(I, Tn, dt, Alphan1, Omegan1, 100*eps);
        % update velocity
        Omegan = Omegan1+dt/2*(Alphan1+Alphan);
        % swap variables for next step
        Rn1=Rn;
        Omegan1=Omegan;
        Alphan1=Alphan;
        t=t+dt;
    end
    self.R = Rn;
    self.Omega = Omegan;
    self.rotint = set(self.rotint,'t',t);
    retobj=self;
end

function Alphan = solve_Alpha(I, Tn, dt, Alphan1, Omegan1, ceps)
    maxi=12;
    Omeganp=Omegan1+dt/2*Alphan1;
    B = I + dt/2*skewmat(Omeganp)*I - skewmat(dt/2*I*(Omeganp));
    Alphan=Alphan1;
    i=0;
    res = I*Alphan - Tn + skewmat(Omeganp+dt/2*Alphan)*I*(Omeganp+dt/2*Alphan);
    while (max(abs(res)) > ceps)
        Alphan = Alphan - (B+(dt/2)^2*(skewmat(Alphan)*I-skewmat(I*(Alphan))))\res;
        res = I*Alphan - Tn + skewmat(Omeganp+dt/2*Alphan)*I*(Omeganp+dt/2*Alphan);
        i=i+1;
        if (i > maxi)
            if (max(abs(res)) > 1e-9)
                warning(['Failed to converge: ||residual||=' num2str(norm(res))]);
            end
            break;
        end
    end
end
    %disp(['Needed ' num2str(i) ' iterations']);
    % omegan = omeganp+dt/2*alphan;
    % err=alphan-(In\(Tn - skewmat(omegan)*In*omegan));
    % if (norm(err) > eps)
    %     warning('wrong solution'); err
    % end