rho=1.21*1e-9;% mass density
c =343*1000;% millimeters per second
bulk= c^2*rho;
L=500;% length of the box, millimeters
A=200; % cross-sectional area of the box
graphics= true;% plot the solution as it is computed?
n=32;%
neigvs=5;
OmegaShift=1000;

[fens,gcells] = T4_cylinderdel(L,sqrt(A)/2,n,1);

prop=property_viscous_fluid(struct('bulk',bulk,'rho',rho));
mater=mater_defor_ss_linviscel_triax (struct('property',prop));
feb = feblock_acoustics (struct (...
    'mater',mater,'gcells',gcells,...
    'integration_rule',tet_rule(1)));

geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
P = field(struct ('name',['P'], 'dim', 1,'nfens',get(geom,'nfens')));
P = numbereqns (P);

S = start (sparse_sysmat, get(P, 'neqns'));
S = assemble (S, acoustic_stiffness(feb, geom, P));
C = start (sparse_sysmat, get(P, 'neqns'));
C = assemble (C, acoustic_mass(feb, geom, P));


[W,Omega]=eigs(get(C,'mat')+OmegaShift*get(S,'mat'),get(S,'mat'),neigvs,'SM');
[Omegas,ix]=sort(diag(Omega)-OmegaShift);
fs=sqrt(Omegas)/(2*pi);

if graphics
        Camera =1.0e+003 * [ -0.8850   -0.8988    1.2071    0.0987   -0.0586    0.1216         0         0    0.0010    0.0107];
        bg=mesh_bdry(gcells);
        bfeb = feblock(struct ('gcells',bg));
        gv=graphic_viewer;
end
w=0*clone(P,'W');
for i=1:neigvs
    analyt_eigenvalue =sqrt((bulk/rho*((i-1)*pi/L) ^ 2))/2/pi;
    eigenvalue_errors(i) =...
        (fs(i)-analyt_eigenvalue)/analyt_eigenvalue;
    disp(['  Eigenvector ' num2str(i) ...
        ' eigenvalue ' num2str(fs(i)) ' versus '...
        num2str(analyt_eigenvalue)]);
    
    w = scatter_sysvec(w, real(W(:,ix(i))));
    
    
    % Plot
    if (graphics)
        gv=reset (gv,[]);
        w = scatter_sysvec(w, W(:,ix(i)));
        wv = get(w,'values');
        dcm=data_colormap(struct ('range',[min(wv),max(wv)], 'colormap',jet));
        colors=map_data(dcm, wv);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        gv=reset (gv,[]);
        camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom,'u',0*geom,'colorfield',colorfield));
        draw_axes (gv,struct('length',20 ));
        pause(0.55);
    end
end