
%%  Timoshenko-Gere frame buckling
% *Under development*

% Parameters:
E=7.2e6;%N/cm^2
nu=0.;
A=6;%cm^2
I=2;%cm^4 
L=120;% cross-sectional dimensions and length of each leg
q=+1;
neigvs=4;


% Mesh
n=6;
% 
% Cross-section parameters of member 1. Note that its  finite elements
% (geometrical cells) are labeled 1.
Parameters=struct ('A',A,'I2',I,'I3',I,'J',2*I,...
    'x1x2_vector',[0 0 1], 'label',1);
% Mesh of member 1
[fens1,gcells1] = Beam_member([[0 L 0];[L L 0]],n,@gcellset_beam3,Parameters);
% Cross-section parameters of member 2. Note that its  finite elements
% (geometrical cells) are labeled 2.
Parameters=struct ('A',A,'I2',I,'I3',I,'J',2*I,...
    'x1x2_vector',[0 0 1], 'label',2);
% Mesh of member 2
[fens2,gcells2] = Beam_member([[L L 0];[L 0 0]],n,@gcellset_beam3,Parameters);
% Merge meshes, but make sure that no nodes are merged by using 
% the tolerance of 0. 
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, 0);
gcells= cat(gcells1,gcells2);
% Set up the link between the nodes. In this case all the translations are 
% the same (spherical joint). 
link.component= [1,2,3]; % spherical joint
% Now find which nodes are at the location where the beam members should be joined.
[link.master, link.slave]= Beam_join(fens,gcells, 1, 2, L/1000);

prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss_beam3(struct ('mater',mater, 'gcells',gcells));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));

% Apply EBC's
% First constrain everything to the xy plane
for ebc_comp=[3,4,5];
    ebc_fenids=1: count(fens);
    ebc_prescribed=[1];
    ebc_val=ebc_fenids*0;
    ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
end
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 L 0])));
ebc_prescribed=[1];
ebc_comp=[];% clamped
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select(fens, struct('box', update_box([],[L 0 0])));
ebc_prescribed=[1];
ebc_comp=[1,2,3,5];% cylindrical joint
ebc_val=0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);

% Number equations
ur  = numbereqns (ur, {link});
neqns=get(ur, 'neqns');

% Assemble the system matrix
K = start (sparse_sysmat, neqns);
K = assemble (K, stiffness(feb, geom, ur));
% Load
% Load
F = start (sysvec, neqns);
fi = force_intensity(struct('magn' ,[0,-q,0]'));
% Create a special FE block just for the application of the load on the
% finite elements of member 1 
lfeb = feblock_defor_ss_beam3(struct ('mater',mater,...
    'gcells',subset(gcells,gcell_select(fens,gcells,struct ('label', 1)))));
evs = distrib_loads(lfeb, geom, ur, fi);
F = assemble (F, evs);
% Solve
ur = scatter_sysvec(ur, K\F);
% get(ur,'values')


% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
scale=90;
u=slice(ur,(1:3),'u');
rot=slice(ur,(4:6),'rot');
draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','none'));
draw_local_coordinates(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'length',25));

xlabel('X');
ylabel('Y');
zlabel('Z');
hold on

draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
draw_axes(gv, struct([]));
