function  spherical_punch_aug_h20
    disp('Spherical punch pushed into a block');

    c =clock; disp(['Started ' num2str(c(4)) ':' num2str(c(5)) ':' num2str(c(6)) ' '])
    % Parameters:
    E=1e3;
    nu= 0.498;
    L= 300;
    W= 300;
    H= 300;
    R=400;
    nsteps = 2;
    center = [L+R, W, H];
    travel =R/4;
    dcenter = [-travel/nsteps,0, 0];
    nedges = 14;
    penalty = 100*E;

    function  gap = get_gap (surface_data,x)
        gap =  norm(center-x)-R;
        if (gap>0), gap = 0; end
    end
    function normal = get_normal(surface_data,x)
        normal=(x-center)/norm(center-x);
    end

    % Mesh
    %     [fens,gcells] = blockH20(L,W,H,nedges,nedges,nedges);
    [fens,gcells] = blockH20(L,W,H,nedges,nedges,nedges);

    bdry_gcells = mesh_bdry(gcells,  []);
    topfacelist =gcell_select(fens,bdry_gcells,struct ('box',[L L 0 W 0 H],'inflate',L/1000));
    cn = connected_nodes(bdry_gcells(topfacelist));
    gap_gcells(1:length(cn)) = deal(gcell_P1);
    for i= 1:length(cn)
        gap_gcells(i)=gcell_P1 (struct('conn',cn(i)));
    end

    % Material
    prop = property_linel_iso (struct('E',E,'nu',nu));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    % Finite element block
    feb = feblock_defor_ss (struct('mater',mater,'gcells',gcells,...
        'integration_rule',gauss_rule(3, 3)));
    gapfeb = feblock_defor_gap (struct('gcells',gap_gcells,...
        'get_gap',@get_gap,'get_normal',@get_normal,'penalty',penalty));
    % Geometry
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    % Define the displacement field
    u   = clone(geom,'u');
    u   = u*0; % zero out
    % Apply EBC's
    ebc_fenids=fenode_select (fens,struct('box',[0, 0, 0, W, 0, H],'inflate',W/1000));
    ebc_prescribed=ebc_fenids*0+1;
    ebc_comp=[];% all components
    ebc_val=ebc_fenids*0;% zero displacement
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    u   = apply_ebc (u);
    % Number equations
    u   = numbereqns (u);
%     K = start (sparse_sysmat, get(u, 'neqns'));
%     ems =cat(2,stiffness(feb, geom, u),stiffness(gapfeb, geom, u));
%     K = assemble (K, ems);
%     perm = symamd(get(K,'mat'));
%     u   = numbereqns(u,perm);
        
    %     h=progressbar(mfilename);
    for step= 1:nsteps
        disp(['Step ' num2str(step)])
        center= center +dcenter;
        u   = u*0; % zero out
    
        tic;
        K = start (sparse_sysmat, get(u, 'neqns'));
        K = assemble (K, cat(2,stiffness(feb, geom, u),stiffness(gapfeb, geom, u)));
        disp(['Stiffness matrix assembly ' num2str(toc)])
        F = start (sysvec, get(u, 'neqns'));
        lm =get(F,'vec');
        %         gv=reset (graphic_viewer,[]);
        for augit= 1:3
            disp([' Iteration ' num2str(augit)])
            F = assemble (start (sysvec, get(u, 'neqns')), gap_loads(gapfeb, geom, u));
            gl =get(F,'vec');
            tic;
%             us = get(K,'mat')\(-lm+gl);%bicg(get(K,'mat'),(-lm+gl), 1e-6, 140, diag(diag (get(K,'mat'))));
            us = bicgstab(get(K,'mat'),(-lm+gl), 1e-6, 200, diag(diag (get(K,'mat'))));
            disp(['      Solve ' num2str(toc)])
        u = scatter_sysvec(u, us);
            lm=lm-gl;
%             draw(feb,gv, struct ('x', geom, 'u', (1)*u,'facecolor','red'));pause(5)
        end
        geom =geom+u;% update geometry
        %         if isempty(progressbar((step/nsteps),h)), return, end;
    end
     c =clock; disp(['Stopped ' num2str(c(4)) ':' num2str(c(5)) ':' num2str(c(6)) ' '])
    % Plot
    gv=graphic_viewer;
    gv=reset (gv,[]);
    scale=1;
    %         draw(feb,gv, struct ('x', geom, 'u', 0*u, 'facecolor','none'));
    draw(feb,gv, struct ('x', geom, 'u', (0)*u,'facecolor','red'));
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    hold on
end
