%%  Straight cantilever, curl up with a moment at the tip
% 
% Cantilever beam turned into a circle by a concentrated moment as the tip.

function CB_cantilever_curlup
    %%
    %
    % Parameters:
    E=1e7;% lb.in^-2
    nu=0.3;
    h= 1; b =1;% in
    Parameters=Beam_cross_section(struct('type','rectangle','d2',h,'d3',b));
    Length=100;% in
    Mmag=2*pi*E*Parameters.I2/Length;%  lb
    nel=18;% number of elements
    utol=1e-6;
    
    % Mesh
    X= [0,0,0;0, Length,0];
    Parameters.x1x2_vector=[0 0 1];
    [fens,gcells] = Beam_member(X,nel,@gcellset_beam3,Parameters);
    basen=fenode_select(fens, struct('box', update_box([],X(1,:))));
    tipn=fenode_select(fens, struct('box', update_box([],X(2,:))));
    
    
    
    % Material
    prop = property_linel_iso (struct('E',E,'nu',nu));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    % Finite element block
    feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));
    % Initialize configuration variables
    geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
    u0=0*geom0;
    Rfield0= field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
    % Define the displacement increment field
    dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));
    
    % Apply EBC's
    ebc_fenids=[basen];
    ebc_prescribed=[1];
    ebc_comp=[];
    ebc_val=0;
    dchi = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    dchi = apply_ebc (dchi);
    % Number equations
    dchi = numbereqns (dchi);
    neqns=get(dchi, 'neqns');
    
    gv=graphic_viewer;
    
    
    load_parameters=0.05:0.05:1;
    tip_displacement=[];
    for load_parameter=load_parameters
        
        % Initialization
        dchi = 0*dchi; % guess
        dchi = apply_ebc(dchi);
        u1 = u0;
        Rfield1=Rfield0;
        nl = nodal_load(struct ('id',tipn,'dir',[4],'magn',[load_parameter*Mmag]));
        
        iter=1;
        while 1
            feb1=feb;  % Make a copy of the state
            F = start (sysvec, neqns);
            F = assemble (F, loads(nl, dchi));
            [feb1,evs] = restoring_force(feb1,geom0,u1,Rfield1,dchi);       % Internal forces
            F = assemble (F, evs);
            K = start (sparse_sysmat, neqns);
            K = assemble (K, cat(2,stiffness(feb1,geom0,u1,Rfield1,dchi),stiffness_geo(feb1,geom0,u1,Rfield1,dchi)));
            dchi = scatter_sysvec(dchi, K\F); % Displacement increment
            % Line search
            R0 = dot(get(F,'vec'),gather_sysvec(dchi));
            F = start (sysvec, neqns);
            F = assemble (F, loads(nl, dchi));
            [feb1,evs] = restoring_force(feb1,geom0,...
                u1+slice(dchi,(1:3),'transl'),update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat')),dchi);       % Internal forces
            F = assemble (F, evs);
            R1 = dot(get(F,'vec'),gather_sysvec(dchi));
            a = R0/R1;
            if ( a<0 )
                eta = a/2 +sqrt((a/2)^2 -a);
            else
                eta =a/2;
            end
            eta=min( [eta, 1.0] );
            u1 = u1 + eta*slice(dchi,(1:3),'transl');   % increment displacement
            Rfield1=update_rotation_field(Rfield1,eta*slice(dchi,(4:6),'rotat'));
            disp(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
            if (max(abs(gather(dchi,[],'values'))) < utol) break; end;                    % convergence check
            iter=iter+1;
        end
        [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);        % final update
        disp(['    Converged for load_parameter=' num2str(load_parameter)]); % pause
        u0 = u1; % update the displacement
        Rfield0=Rfield1; % update the rotations
        utip=gather(u1,get(nl, 'id'),'values');
        disp(['Tip deflection =' num2str(utip(1:3)','%27.15e, %27.15e, %27.15e\n')])
        tip_displacement=[tip_displacement;utip(1:3)'];
        
        %%
        % Graphic display of the mesh and the deformed beam
        gv=reset (gv,[]);
        scale=1;
        draw(feb, gv, struct ('x', geom0, 'u', [], 'R', [], 'facecolor','none'));
        draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor',[1,1,1]/2));
        %     draw_local_coordinates(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', 0*rot, 'length',25));
        
        xlabel('X');
        ylabel('Y');
        zlabel('Z');
        hold on
        
        %     draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
        draw_axes(gv, struct([]));
        camset (gv, [838.5997 -100.2913  238.4621    0.3923 39.9765   26.5675         0         0 1.0000    5.0831]);
        pause(1)
    end
    
end