%% Spinning "top" made up of four bars
%

function f
% Parameters:
E=30002*1000;%30002ksi
nu=0.3;
radius_internal=2.5/2;%in
radius_external=6.25/2;%in
L=10*12;% in
g=32.17*12; % in/second^2
rho=490/12^3/g; %lb/ft^3
Omegaini=13;
q=[0;0;-32];% 1200 lbf/ft
graphics= ~false;
dt=1/Omegaini/10;
tend=400*dt;
Newmarkg = 1/2; Newmarkb = 1/4*(1/2+Newmarkg)^2;
% Newmarkg = 0.6; Newmarkb = 1/4*(1/2+Newmarkg)^2;
maxit=12;
Report =@(x) (disp(x) );
%%
% Cross-sectional properties
Parameters=Beam_cross_section(struct('type','hollow_circle','Rint',radius_internal,'Rext',radius_external));
Parameters.x1x2_vector=[1 0 0];


%%
%
n=2;
tolerance=L/n/100;
a= [1,0.2,0]';
[fens,gcells]  = Beam_member([[0 0 0];L*a'],n,@gcellset_beam3,Parameters);
[fens1,gcells1]  = Beam_member([[0 0 0];(L*rotmat(120/180*pi*[1;0;0])*a)'],n,@gcellset_beam3,Parameters);
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tolerance);
gcells= cat(gcells1,gcells2);
[fens1,gcells1]  = Beam_member([[0 0 0];(L*rotmat(240/180*pi*[1;0;0])*a)'],n,@gcellset_beam3,Parameters);
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tolerance);
gcells= cat(gcells1,gcells2);
basen=fenode_select(fens, struct('box', update_box([],[0 0 0])));
tipn=fenode_select(fens, struct('box', update_box([],L*a')));

%%
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%%
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));


%%
% Initialize configuration variables
geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
u0=0*geom0;
Rfield0= field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));

% Apply EBC's
dchi  = set_ebc(dchi, basen, 1, [1,2,3], 0);
dchi  = apply_ebc (dchi);

% Number equations
dchi  = numbereqns (dchi);
neqns=get(dchi, 'neqns');


if graphics, gv=graphic_viewer; end

% Now comes the nonlinear solution
dchi = dchi*0; % zero out the displacement
v = dchi*0; % zero out the Velocity
xyz = gather(geom0,(1:count(fens)),'values','noreshape');
vxyz = gather(v,(1:count(fens)),'values','noreshape');
vxyz =[xyz*skewmat([-Omegaini, 0, 0]), Omegaini+zeros(count(fens),1),zeros(count(fens),2)];
v= scatter (v,(1:count(fens)),vxyz);
a = dchi*0; % zero out the Acceleration
utol = 1e-13*get(dchi, 'neqns');

K = start (sparse_sysmat, get(dchi, 'neqns'));
M = start (sparse_sysmat, get(dchi, 'neqns'));
G = start (sparse_sysmat, get(dchi, 'neqns'));

fi = force_intensity(struct('magn' ,q));

t=0; %
step=0;
while (t <= tend)
    t = t + dt;
    Report(['Time ' num2str(t) ]); % pause
    % Initialization
    dchi = apply_ebc(dchi);% Apply boundary conditions
    u1 = u0; % guess
    Rfield1=Rfield0; % guess
    stepdchi = 0*apply_ebc(dchi);% Total increment in current step
    a1 = -(1/Newmarkb/dt)*v -(1/2-Newmarkb)/Newmarkb*a;
    v1 = v + dt*((1-Newmarkg)*a + Newmarkg*a1);
    dchipv =gather_sysvec(dt*v +(dt^2/2*(1-2*Newmarkb))*a);
    vpv =gather_sysvec(v +(dt*(1-Newmarkg))*a);
    
    iter=1;
    while 1
        feb1=feb;
        F = start (sysvec, get(dchi, 'neqns'));
        F = assemble (F, distrib_loads_global(feb,geom0,u1,Rfield1,dchi, fi));
        [feb,evs] = restoring_force(feb,geom0,u1,Rfield1,dchi);       % Internal forces
        F = assemble (F, evs);
        K =assemble(K,cat(2,stiffness(feb,geom0,u1,Rfield1,dchi),stiffness_geo(feb,geom0,u1,Rfield1,dchi)));
        Km=get(K, 'mat');
        M =assemble (M,mass(feb,geom0,u1,Rfield1,dchi));
        Mm=get(M, 'mat');
        G =assemble (G,coriolis(feb,geom0,u1,Rfield1,v1,dchi));
        Gm=get(G, 'mat');
        rhs=get(F,'vec')...
            +Mm*((-1/(Newmarkb*dt^2))*gather_sysvec(stepdchi)+(1/(Newmarkb*dt^2))*dchipv)...
            +Gm*((-Newmarkg/Newmarkb/dt)*gather_sysvec(stepdchi)+(Newmarkg/Newmarkb/dt)*dchipv-vpv);
        dchi = scatter_sysvec(dchi, (Km+(Newmarkg/Newmarkb/dt)*Gm+(1/(Newmarkb*dt^2))*Mm)\rhs); % Displacement increment
        u1 = u1 + slice(dchi,(1:3),'transl');   % increment displacement
        stepdchi = stepdchi + dchi;
        v1 = v1 + (Newmarkg/Newmarkb/dt)*dchi;
        a1 = a1 + (1/Newmarkb/dt^2)*dchi;
        Rfield1=update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat'));
                    Report(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
        if (max(abs(gather(dchi,[],'values'))) < utol) break; end;% convergence check
        if (iter >maxit)% bailout for failed convergence
            error( [' Possible failed convergence']);
        end
        K = reinitialize (K);
        M = reinitialize (M);
        G = reinitialize (G);
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);        % final update
    %         Report(['    Converged for t=' num2str(t)]); % pause
    u0 = u1;       % update the displacement
    Rfield0=Rfield1; % update the rotations
    v = v1;       % update the velocities
    a = a1;       % update the accelerations
    
    if graphics
        gv=reset(gv,[]);
        draw(feb, gv, struct ('x', geom0, 'u', [], 'R', [], 'facecolor','r'));
        draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','g'));
        %         draw_local_coordinates(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'length',12.5));
                camset(gv, [-536.1447 -716.2187  478.7955   16.9688    4.6129  -45.7783         0         0    1.0000   10.3396]); 
        drawnow;
    else
        
    end
    step=step+1;
end

