disp('Thin ring');

% Parameters:
E=1e7;
nu=0.25;
d= 1;
rin=10;
rex=rin+d;
rim=(rin+rex)/2;
scale=500;
F = 20;
htol= d/100;
graphics =  ~~true;

Meshf=@t3annulus;
integration_rule = simplex_nq_rule(struct('dim',2,'order',1));
febf =@feblock_defor_ss_nice;
Style ='b*-';

%     Meshf=@annulus;
%     integration_rule = gauss_rule(2, 2);
%     febf =@feblock_defor_ss;
%     Style ='gs-';
% 
%  Meshf=@annulus;
%     integration_rule = gauss_rule(2, [2,2]);
%     febf =@feblock_defor_ss_sri;
%     Style ='ro-';

for n= 1:9
    % Mesh
    [fens,gcells] = Meshf(rin,rex, n, 3*n, 1.0);
    propm = property_linel_iso(struct('E',E,'nu', nu));
    mater = mater_defor_ss_linel_biax (struct ('property',propm,'reduction','stress'));
    feb = febf(struct ('mater',mater, 'gcells',gcells, 'integration_rule',integration_rule));
    % Geometry
    geom = field(struct ('name',['geom'], 'dim', 2, 'fens',fens));
    % Define the displacement field
    u   = clone(geom,'u');
    u   = u*0; % zero out
    % Apply EBC's
    nl=fenode_select (fens,struct ('box',[0 0 0 100*rex],'inflate',htol));
    u   = set_ebc(u, nl, nl*0+1, nl*0+1, nl*0);
    u   = apply_ebc (u);
    nl=fenode_select (fens,struct ('box',[0 100*rex 0 0],'inflate',htol));
    u   = set_ebc(u, nl, nl*0+1, nl*0+2, nl*0);
    u   = apply_ebc (u);
    % Number equations
    u   = numbereqns (u);
    % Assemble the stiffness matrix
    S = start (sparse_sysmat, get(u, 'neqns'));
    S = assemble (S, stiffness(feb, geom, u));
    S = finish (S);
    % Load
    nl=fenode_select (fens,struct ('box',[0 0 rex rex],'inflate',htol));
    L = start (sysvec, get(u, 'neqns'));
    L = assemble (L, loads(nodal_load(struct('id',nl,'dir',2,'magn',-F/2)), u));
    L = finish(L);
    % Solve
    u = scatter_sysvec(u, get(S,'mat')\get(L,'vec'));
    nl=fenode_select (fens,struct ('box',[0 0 rin rin],'inflate',htol));
    uA = gather (u,nl,'values');
    Iz =rim^3*log ((2*rim+ d)/(2*rim- d)) -rim^ 2*d;
    uAex =F*rim ^3/(4*E*Iz)*(pi/2 -4/pi);
    disp (['Normalized displacement under the load: ' num2str((-uA(2))/uAex*100) '%'])

    % Plot
    if graphics
        gv=graphic_viewer;
        % for i=1:length(fens)
        %     draw(fens(i),gv, struct ('x', geom, 'u', 0*u, 'facecolor','blue'));
        % end
        gv=reset (clear (gv),struct('limits',[0 1.05*rex 0 1.05*rex]));
        draw(feb,gv, struct ('x', geom, 'u',u, 'facecolor','none'));
        fld = field_from_integration_points(feb, geom, scale*u, [], 'Cauchy', 2);
        nvals=get(fld,'values');
        dcm=data_colormap(struct ('range',[min(nvals),max(nvals)], 'colorimap',jet));%[min(nvals),max(nvals)]
        colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
        draw(feb, gv, struct ('x',geom,'u', scale*u, 'colorfield',colorfield, 'shrink',1));
        view (2)
    end
end