% Thick plate, small displacement, small strain analysis
disp('Timoshenko: clamped square plate with center load');
% !  Timoshenko: clamped square plate with center load
% ! w_max = 0.0056 * P * a^2 / (E * h^3 / (12 * (1 - nu^2))) =Hanover
% County's bumpy
% !     2.44608 for P=10, a=2, E=1e9, h=0.001, nu=0.3
% ! full model (not reduced by symmetry) -- 289 (displ) + 68 (lagr) unknowns
% ! reduced -- 81 (displ) + 54 (lagr)

% Parameters:
P=2; a=2; E=1e9; h=a/100; nu=0.2999;
stabfact = 0.0000;
w_max = 0.0056 * P * a^2 / (E * h^3 / (12 * (1 - nu^2)));
scale = 0.5/w_max;

nas = (2:8);
nas = 4;
nts = 1;

graphics = ~false; % graphic output

eix=1;
clear eltyd

% % NICE tetrahedron
eltyd(eix).mf =@t4block;
eltyd(eix).subdiv =2;
eltyd(eix).blf =@feblock_defor_ss_nicer;
eltyd(eix).integration_rule= simplex_nq_rule (3);
eltyd(eix).surface_integration_rule=tri_rule(1);
eltyd(eix).styl='c*-';
eix=eix+1;
%
% % NICE  hexahedron
% eltyd(eix).mf =@block;
% eltyd(eix).subdiv =2;
% eltyd(eix).blf =@feblock_defor_ss_nicer;
% eltyd(eix).integration_rule= tensprod_nq_rule (struct('dim',3,'order',1));
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='cd-';
% eix=eix+1;
%
% Full integration 27-node hexahedron
% eltyd(eix).mf =@blockH27;
% eltyd(eix).subdiv =1;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,3);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='k<-';
% eix=eix+1;
%
% % NICE  27-node hexahedron
% eltyd(eix).mf =@blockH27;
% eltyd(eix).subdiv =1;
% eltyd(eix).blf =@feblock_defor_ss_nicer;
% eltyd(eix).integration_rule=tensprod_nq_rule (struct('dim',3,'order',2));
% eltyd(eix).surface_integration_rule=tensprod_nq_rule (struct('dim',2,'order',2));
% eltyd(eix).styl='ks-';
% eix=eix+1;
%
% % Reduced integration H20
% eltyd(eix).mf =@blockH20;
% eltyd(eix).subdiv =1;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,2);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='ro-';
% eix=eix+1;



% Full integration H20
% eltyd(eix).mf =@blockH20;
% eltyd(eix).subdiv =1;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,3);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='b*-';
% eix=eix+1;

% T10 tetrahedron
% eltyd(eix).mf =@t4blockT10;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=tet_rule (4);
% eltyd(eix).surface_integration_rule=tri_rule(3);
% eltyd(eix).styl='gd-';
% eix=eix+1;

% Selective reduced integration hexahedron
% eltyd(eix).mf =@block;
% eltyd(eix).blf =@feblock_defor_ss_sri;
% eltyd(eix).integration_rule=gauss_rule (3,[1,2]);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='md-';
% eix=eix+1;

% Full integration hexahedron
% eltyd(eix).mf =@block;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,2);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='mx--';
% eix=eix+1;



for eix = 1:length(eltyd)
    normalized_deflections = [];
    neqns = [];
    for nt=nts
        for na=nas
            [fens,gcells]= feval (eltyd(eix).mf, a/2,a/2,h,na,na,nt);
            % Material
            prop = property_linel_iso (struct('E',E,'nu',nu));
            mater = mater_defor_ss_linel_triax (struct('property',prop));
            % Finite element block
            feb = feval (eltyd(eix).blf, struct ('mater',mater, 'gcells',gcells, 'integration_rule',eltyd(eix).integration_rule));
            % Geometry
            geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
            % Define the displacement field
            u   = 0*geom; % zero out
            % Apply EBC's
            ebc_fenids=fenode_select (fens,struct('box',[0 0 0 a/2 0 h],'inflate',h/100));
            ebc_prescribed=ebc_fenids*0+1;
            ebc_comp=ebc_fenids*0+1;
            ebc_val=ebc_fenids*0;
            u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
            ebc_fenids=fenode_select (fens,struct('box',[0 a/2 0 0 0 h],'inflate',h/100));
            ebc_prescribed=ebc_fenids*0+1;
            ebc_comp=ebc_fenids*0+2;
            ebc_val=ebc_fenids*0;
            u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
            ebc_fenids=fenode_select (fens,struct('box',[0 a/2 a/2 a/2 0 h],'inflate',h/100));
            ebc_prescribed=ebc_fenids*0+1;
            ebc_comp=[];
            ebc_val=ebc_fenids*0;
            u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
            ebc_fenids=fenode_select (fens,struct('box',[a/2 a/2 0 a/2 0 h],'inflate',h/100));
            ebc_prescribed=ebc_fenids*0+1;
            ebc_comp=[];
            ebc_val=ebc_fenids*0;
            u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
            u   = apply_ebc (u);
            % Number equations
            u   = numbereqns (u);
            % Assemble the system matrix
            ems = stiffness(feb, geom, u);
            K = start (sparse_sysmat, get(u, 'neqns'));
            K = assemble (K, ems);
            % Load
            F = start (sysvec, get(u, 'neqns'));
            corn=fenode_select (fens,struct('box',[0 0 0 0 0 h],'inflate',h/100));
            evs = loads(nodal_load(struct ('id',corn,'dir',[3],'magn',[-P/4/length(corn)])), u);
            F = assemble (F, evs);
            % Solve
            u = scatter_sysvec(u, get(K,'mat')\get(F,'vec'));
            ucorn= gather(u, corn,'values','noreshape');
            nd=abs(mean(ucorn(:,3))/w_max)
            %toc
            normalized_deflections = [normalized_deflections,nd];
            neqns = [neqns get(u, 'neqns')];
            if graphics
                gv=graphic_viewer;
                gv=reset (gv,[]);
                draw(gcells, gv, struct ('x',geom,'u', scale*u, 'facecolor','none'));
                camset(gv, [-3.3445,-4.2190,3.1679,0.3162,0.5517, -0.3039 , 0, 0,1.0000,10.3396]);
            end
        end
    end
    if  ~graphics
        plot(neqns,abs(normalized_deflections-1),eltyd(eix).styl,'linewidth',3); hold on
        set(gca,'XScale','log')
        set(gca,'YScale','log')
        %                    set(gca,'XLim',[100,1e4])
        %                     set(gca,'YLim',[1e-3,1.0])
        grid on
        figure (gcf); pause (1)
    end
end