function perforated_strip2
    disp('Example Perforated_strip: small strain nonlinear analysis');
    % Parameters:
    E=7000;
    nu=0.3;
    sigma_y=24.3;
    L= 360; % Length of the beam
    W = 10; % Thickness of the plate
    H = 200; % Width
    R= 100;
    nL=8;nH=4;nR=4;
    magn=0.75*2*sigma_y/3;
    scale=20;
    stressscale=scale/20;
    epscale=5*scale;
    nincr =3;
    utol = 10e-7;
    graphics = true;

    % Mesh
    [fens,gcells]=elliphole(R,R,L,H,nL,nH,nR,[]);
    [fens,gcells] = extrudeq4(fens,gcells,1,@(x,i)([x,0]+[0,0,W*i]));
    [fens,gcells] = H8_to_H20(fens,gcells);
    % Material
    prop = property_plastic_perf_j2_iso(struct('E',E,'nu',nu,'sigma_y',sigma_y));
    mater = mater_ss_plastic_perf_j2_triax(struct('property',prop));
    % Finite element block
    feb = feblock_defor_ss_nonlinear (struct ('mater',mater, 'gcells',gcells,...
        'integration_rule',gauss_rule(3,3)));
    % Geometry
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    % Define the displacement field
    u   = 0*geom; % zero out
    Center_fenids=fenode_select (fens,struct('box',[L,L,W/2, W/2,-Inf,Inf],'inflate',1/1000));
    % Apply EBC's
    ebc_fenids=fenode_select (fens,struct('box',[0,0,-Inf,Inf,-Inf,Inf],'inflate',1/1000));
    ebc_prescribed=ones(1,length (ebc_fenids));
    ebc_comp=ones(1,length (ebc_fenids))*1;
    ebc_val=ebc_fenids*0;
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    ebc_fenids=fenode_select (fens,struct('box',[-Inf,Inf,0,0,-Inf,Inf],'inflate',1/1000));
    ebc_prescribed=ones(1,length (ebc_fenids));
    ebc_comp=ones(1,length (ebc_fenids))*2;
    ebc_val=ebc_fenids*0;
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    ebc_fenids=fenode_select (fens,struct('box',[-Inf,Inf,-Inf,Inf,0,0],'inflate',1/1000));
    ebc_prescribed=ones(1,length (ebc_fenids));
    ebc_comp=ones(1,length (ebc_fenids))*3;
    ebc_val=ebc_fenids*0;
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    u   = apply_ebc (u);
    % Number equations
    u   = numbereqns (u);
    % Assemble the system matrix
    ems = stiffness(feb, geom, u, u);
    K = start (dense_sysmat, get(u, 'neqns'));
    K = assemble (K, ems);
    % Load
    bdry_gcells = mesh_bdry(gcells, []);
    bcl = gcell_select(fens, bdry_gcells, ...
        struct ('box',[L,L,-Inf,Inf,-Inf,Inf],'inflate',H/1000));
    lfeb = feblock_defor_ss(struct ('mater',mater, 'gcells',subset(bdry_gcells,bcl),...
        'integration_rule',gauss_rule(2,2)));

    % Now comes the nonlinear solution
    tup = 1;
    u = u*0; % zero out the displacement

    if (graphics)gv=graphic_viewer;end

    Center_deflection = [];
    t=0; % time=load magnitude
    incr=1;
    while (incr <= nincr)
        t = t + tup / nincr;
        % Initialization
        u1 = u; % guess
        u1 = apply_ebc(u1);
        du = 0*u; % this will hold displacement increment
        du = apply_ebc(du);
        iter=1;
        du0norm = [];
        while 1
            feb1=feb;                                           % Make a copy of the state
            fi= force_intensity(struct ('magn',[t*magn;0; 0;]));
            F = start (sysvec, get(u, 'neqns'));
            F = assemble (F, distrib_loads(lfeb, geom, u1, fi,2));
            [Ignore,evs] = restoring_force(feb1,geom,u1,u);       % Internal forces
            F = assemble (F, evs);
            K = start (sparse_sysmat, get(u, 'neqns'));
            K = assemble (K, stiffness(feb1, geom, u1,u));
            du = scatter_sysvec(du, K\F);
            u1 = u1 + du;                                       % increment displacement
            dunorm =norm(du);
            if (isempty(du0norm)) du0norm =dunorm;end
            disp(['   Iteration ' num2str(iter) ': ||du||/||du0||=' num2str(dunorm/du0norm)]);
%             if (graphics)
%                 id.comp= 1;
%                 id.container=-Inf;
%                 id=inspect_integration_points(feb, geom, u1, [],...
%                     (1:length (gcells)), struct ('output',['equiv_pl_def']),...
%                     @mx,id);
%                 max_equiv_pl_def=id.container
%                 id.container=Inf;
%                 id=inspect_integration_points(feb, geom, u1, [], ...
%                     (1:length (gcells)), struct ('output',['equiv_pl_def']),...
%                     @mn,id);
%                 min_equiv_pl_def =id.container
%  fld = field_from_integration_points(feb, geom, u1, [], 'equiv_pl_def',1);
%                 gv=reset(clear(gv,[]),[]);
%                 camset (gv,1.0e+003 *[ -2.1416   -1.4296    3.3375    0.1981    0.1191   -0.0063    0.0006    0.0004    0.0006 0.0039]);
%                 dcm=data_colormap(struct ('range',[min_equiv_pl_def,max_equiv_pl_def], 'colormap',jet));
%                 draw(feb,gv, struct ('x', geom, 'u', +scale*u1,'colorfield',fld, 'edgecolor','black'));
% drawnow; pause(0.1)
%             end
            Center_u =gather (u,Center_fenids,'values','noreshape');

            if (dunorm < utol*du0norm) break; end;                    % convergence check
            iter=iter+1;
        end
        [feb,ignore] = restoring_force(feb1,geom,u1,u);        % final update
        u = u1;                                               % update the displacement
        if graphics
            gv=reset(clear(gv,[]),[]);
            camset (gv,1.0e+003 *[ -2.1416   -1.4296    3.3375    0.1981    0.1191   -0.0063    0.0006    0.0004    0.0006 0.0039]);
            fld = field_from_integration_points(feb, geom, u1, [], 'equiv_pl_def',1);           
            dcm=data_colormap(struct ('range',[0,0.005], 'colormap',jet));
            colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, get(fld,'values'))));
            draw(feb,gv, struct ('x', geom,'u',scale*u, 'colorfield',fld));
            %             draw_integration_points(feb,gv,struct ('x',geom,'u',u,'u_displ',scale*u, 'scale',epscale,'quantity',['equiv_pl_def'],'component',1,'data_cmap', dcm));
            title (['Increment ' num2str(incr) ': Converged for t=' num2str(t) ])
            drawnow; pause(0.1)
        end
        disp(['Increment ' num2str(incr) ': Converged for t=' num2str(t) ]); % pause
        Center_deflection = [Center_deflection,mean(Center_u(:,3))];
        incr = incr + 1;
    end
    Center_deflection
%     camget (gv)
end

function id= mn(id,out,xyz,pc)
    id.container=min(out(id.comp), id.container);
end

function id= mx(id,out,xyz,pc)
    id.container=max(out(id.comp), id.container);
end
%draw(gmlin, 'dataidx', 3, 'scale', scale, 'displace_by', [1 2 3], 'facecolor', 'none')